"Virtualenv utils"
import sys
import os
import subprocess
import virtualenv
import shutil
from os import path

def check_ve(project_root, argv):
    VE_ROOT = path.join(project_root, '.ve')
    VE_TIMESTAMP = path.join(VE_ROOT, 'timestamp')
    REQUIREMENTS = path.join(project_root, 'requirements.pip')

    envtime = path.exists(VE_ROOT) and path.getmtime(VE_ROOT) or 0
    envreqs = path.exists(VE_TIMESTAMP) and path.getmtime(VE_TIMESTAMP) or 0
    envspec = path.getmtime(REQUIREMENTS)

    def go_to_ve(ve_root):
        # going into ve
        if not ve_root in sys.prefix:
            retcode = 3
            while retcode == 3:
                env = os.environ
                if sys.platform == 'win32':
                    python = path.join(VE_ROOT, 'Scripts', 'python.exe')
                elif sys.platform == 'darwin':
                    # temporary fix for broken virtualenv in macports
                    import airy
                    env["PYTHONPATH"] = "%s:" % path.join(path.abspath(path.dirname(airy.__file__)), '..') + env.get('PYTHONPATH', '')
                    python = path.join(VE_ROOT, 'bin', 'python')
                else:
                    python = path.join(VE_ROOT, 'bin', 'python')
                try:
                    retcode = subprocess.call([python, path.join(project_root, 'manage.py')] + argv[1:])
                except KeyboardInterrupt:
                    retcode = 1
            sys.exit(retcode)

    update_ve = 'update_ve' in argv
    if update_ve or envtime < envspec or envreqs < envspec:
        if update_ve:
            # install ve
            if envtime < envspec:
                if path.exists(VE_ROOT):
                    shutil.rmtree(VE_ROOT)
                virtualenv.logger = virtualenv.Logger(consumers=[])
                virtualenv.create_environment(VE_ROOT, site_packages=True)

            go_to_ve(VE_ROOT)

            # check requirements
            if update_ve or envreqs < envspec:
                import pip
                pip.main(initial_args=['install', '-r', REQUIREMENTS, '--upgrade'])
                file(VE_TIMESTAMP, 'w').close()
            sys.exit(0)
        else:
            print "VirtualEnv need to be updated"
            print "Run ./manage.py update_ve"
            sys.exit(1)

    go_to_ve(VE_ROOT)



from datetime import datetime
from hashlib import md5

from users.models import User, Session
from mongoengine.queryset import DoesNotExist


def login(user, raw_password, handler):
    session_key = md5(user.username + raw_password)
    handler.set_secure_cookie("session_key", session_key.hexdigest())

    session = Session.objects.get_or_create(user=user)[0]
    session.session_key = session_key.hexdigest()
    session.save()

    user.last_login = datetime.now()
    user.save()


def authenticate(password, username=None, email=None):
    try:
        if not email:
            user = User.objects.get(username=username)
        else:
            user = User.objects.get(email=email)
        if not user.check_password(password):
            return None
        return user
    except DoesNotExist:
        return None


def get_current_user(handler, *args, **kwargs):
    session_key = handler.get_secure_cookie("session_key")

    if not session_key:
        return None
    try:
        session = Session.objects.get(session_key=session_key)
        return session.user
    except DoesNotExist:
        return None
    return None



from tornado.escape import to_unicode
from airy.utils.html import strip_tags
import logging
import re

_URL_RE = re.compile(ur"""\b((?:([\w-]+):(/{1,3})|www[.])(?:(?:(?:[^\s&()]|&amp;|&quot;)*(?:[^!"#$%&'()*+,.:;<=>?@\[\]^`{|}~\s]))|(?:\((?:[^\s&()]|&amp;|&quot;)*\)))+)""")

def linebreaks(text):
    """
    Turns every new-line ("\n") into a "<br />" HTML tag.
    """
    return to_unicode(text).replace('\n', '<br />')

def linkify(text, shorten=False, extra_params="",
            require_protocol=False, permitted_protocols=["http", "https"]):
    """Converts plain text into HTML with links.

    For example: ``linkify("Hello http://tornadoweb.org!")`` would return
    ``Hello <a href="http://tornadoweb.org">http://tornadoweb.org</a>!``

    Parameters:

    shorten: Long urls will be shortened for display.

    extra_params: Extra text to include in the link tag,
        e.g. linkify(text, extra_params='rel="nofollow" class="external"')

    require_protocol: Only linkify urls which include a protocol. If this is
        False, urls such as www.facebook.com will also be linkified.

    permitted_protocols: List (or set) of protocols which should be linkified,
        e.g. linkify(text, permitted_protocols=["http", "ftp", "mailto"]).
        It is very unsafe to include protocols such as "javascript".
    """
    if extra_params:
        extra_params = " " + extra_params.strip()

    def make_link(m):
        url = m.group(1)
        proto = m.group(2)
        if require_protocol and not proto:
            return url  # not protocol, no linkify

        if proto and proto not in permitted_protocols:
            return url  # bad protocol, no linkify

        href = m.group(1)
        if not proto:
            href = "http://" + href   # no proto specified, use http

        params = extra_params

        # clip long urls. max_len is just an approximation
        max_len = 30
        if shorten and len(url) > max_len:
            before_clip = url
            if proto:
                proto_len = len(proto) + 1 + len(m.group(3) or "")  # +1 for :
            else:
                proto_len = 0

            parts = url[proto_len:].split("/")
            if len(parts) > 1:
                # Grab the whole host part plus the first bit of the path
                # The path is usually not that interesting once shortened
                # (no more slug, etc), so it really just provides a little
                # extra indication of shortening.
                url = url[:proto_len] + parts[0] + "/" +\
                      parts[1][:8].split('?')[0].split('.')[0]

            if len(url) > max_len * 1.5:  # still too long
                url = url[:max_len]

            if url != before_clip:
                amp = url.rfind('&')
                # avoid splitting html char entities
                if amp > max_len - 5:
                    url = url[:amp]
                url += "..."

                if len(url) >= len(before_clip):
                    url = before_clip
                else:
                    # full url is visible on mouse-over (for those who don't
                    # have a status bar, such as Safari by default)
                    params += ' title="%s"' % href

        return u'<a href="%s"%s>%s</a>' % (href, params, url)

    # The regex is modified to avoid character entites other than &amp; so
    # that we won't pick up &quot;, etc.
    return _URL_RE.sub(make_link, text)

def sanitize(text):
    """
    Cleans up user input from potentially dangerous things,
    such as <script>, <img src="javascript:..." />, <a href="javascript:.."> etc.

    Please note, this function is **not 100% safe**.

    If want to ensure the input is safe it's best to just escape all HTML.

    If you decide to use ``markup,sanitize()`` make sure you have html5lib available on your system/project.
    """
    try:
        from airy.core import sanitizer
        return sanitizer.clean_html(text)
    except ImportError:
        logging.error("You need html5lib in order to use sanitize")
        return "ERROR: You need html5lib in order to use sanitize"

def truncate(text, limit=80):
    return text[:limit]+'...'



import os
import sys
import time
import signal
import threading
import atexit
import Queue

_interval = 1.0
_times = {}
_files = []

_running = False
_queue = Queue.Queue()
_lock = threading.Lock()

def _restart_parent(*args, **kwargs):
    sys.exit(3)

signal.signal(signal.SIGINT, _restart_parent)

def _restart(path):
    _queue.put(True)
    prefix = 'monitor (pid=%d):' % os.getpid()
    #print >> sys.stderr, '%s Change detected to \'%s\'.' % (prefix, path)
    #print >> sys.stderr, '%s Triggering process restart.' % prefix
    os.kill(os.getpid(), signal.SIGINT)

def _modified(path):
    try:
        # If path doesn't denote a file and were previously
        # tracking it, then it has been removed or the file type
        # has changed so force a restart. If not previously
        # tracking the file then we can ignore it as probably
        # pseudo reference such as when file extracted from a
        # collection of modules contained in a zip file.

        if not os.path.isfile(path):
            return path in _times

        # Check for when file last modified.

        mtime = os.stat(path).st_mtime
        if path not in _times:
            _times[path] = mtime

        # Force restart when modification time has changed, even
        # if time now older, as that could indicate older file
        # has been restored.

        if mtime != _times[path]:
            return True
    except:
        # If any exception occured, likely that file has been
        # been removed just before stat(), so force a restart.

        return True

    return False

def _monitor():
    while 1:
        # Check modification times on all files in sys.modules.

        for module in sys.modules.values():
            if not hasattr(module, '__file__'):
                continue
            path = getattr(module, '__file__')
            if not path:
                continue
            if os.path.splitext(path)[1] in ['.pyc', '.pyo', '.pyd']:
                path = path[:-1]
            if _modified(path):
                return _restart(path)

        # Check modification times on files which have
        # specifically been registered for monitoring.

        for path in _files:
            if _modified(path):
                return _restart(path)

        # Go to sleep for specified interval.

        try:
            return _queue.get(timeout=_interval)
        except:
            pass

_thread = threading.Thread(target=_monitor)
_thread.setDaemon(True)

def _exiting():
    try:
        _queue.put(True)
    except:
        pass
    try:
        _thread.join()
    except RuntimeError:
        pass

atexit.register(_exiting)

def track(path):
    if not path in _files:
        _files.append(path)

def start(interval=1.0):
    global _interval
    if interval < _interval:
        _interval = interval

    global _running
    _lock.acquire()
    if not _running:
        prefix = 'monitor (pid=%d):' % os.getpid()
        # print >> sys.stderr, '%s Starting change monitor.' % prefix
        _running = True
        _thread.start()
    _lock.release()



"Base engine"
import tornado.web
from tornado import template, autoreload
from tornadio2 import SocketServer, TornadioRouter
from airy.core.conf import settings, _preconfigure
from airy.core import web
import os

AIRY_ROOT = os.path.join(os.path.abspath(os.path.dirname(__file__)), '../')

def execute(project_root, argv):
    _preconfigure(project_root, argv=argv[1:])

    if len(argv) <= 1:
        print 'Please provide a command.'
    elif argv[1] == 'run' or argv[1] == 'runserver':
        run(project_root)
    elif argv[1] == 'shell':
        shell()
    elif argv[1] == 'help':
        import tornado.options
        tornado.options.print_help()
    else:
        print "Error: unknown command '%s'" % argv[1]


def shell():
    from IPython import embed
    embed()

def run(project_root):
    serverurls = []
    servermodels = []

    # add Airy static file handler
    serverurls.extend([
        (r"/airy/(.*)", tornado.web.StaticFileHandler, {"path": os.path.join(AIRY_ROOT, 'static')})
    ])

    # add Airy core handler
    serverurls.extend(web.core_router.urls)

    # add application handlers
    for appname in settings.installed_apps:
        # add app urls
        appurls = __import__('%s.urls' % appname, fromlist=['%s.url'%appname])
        urlpatterns = getattr(appurls, 'urlpatterns')
        serverurls.extend(urlpatterns)
        try:
            models = __import__('%s.models' % appname, fromlist=['%s.models'%appname])
            servermodels.append(models)
        except ImportError:
            pass

    # restart on code change
    for root,dirs,files in os.walk(settings.template_path):
        for x in files:
            if os.path.splitext(x)[1].lower() == '.html': # track templates
                autoreload._watched_files.add(os.path.join(root,x))

    application = tornado.web.Application(serverurls, **settings.__dict__)
    settings.application = web.site.application = application
    web.site.loader = template.Loader(settings.template_path)
    
    SocketServer(application)




from datetime import datetime
from hashlib import md5

from airy.core.db import *


class Education(Document):
    university = StringField(max_length=512)
    degree = StringField(max_length=512)
    major = StringField(max_length=512)
    description = StringField(required=False)
    start_date = DateTimeField()
    end_date = DateTimeField()

    def __unicode__(self):
        return self.university


class Experience(Document):
    employer = StringField(max_length=512)
    position = StringField(max_length=512)
    description = StringField(required=False)
    start_date = DateTimeField()
    end_date = DateTimeField()

    def __unicode__(self):
        return self.employer


class Service(Document):
    name = StringField(max_length=512)
    date_created = DateTimeField(default=datetime.now)
    providers_num = IntField(default=0)

    def __unicode__(self):
        return self.name


class PasswordResetToken(Document):
    expired = DateTimeField()
    token = StringField(max_length=128)


class User(Document):
    username = StringField(max_length=30)
    password = StringField(max_length=128)
    first_name = StringField(max_length=30)
    last_name = StringField(max_length=30)
    email = StringField(max_length=30)
    last_login = DateTimeField(default=datetime.now)
    date_joined = DateTimeField(default=datetime.now)
    picture_url = StringField(max_length=512, default='')
    picture_height = IntField(default=0)
    picture_width = IntField(default=0)
    headline = StringField(max_length=512, default='')
    country = StringField(max_length=512, required=False)
    state = StringField(max_length=512, required=False)
    city = StringField(max_length=512, required=False)
    bio = StringField(default='')
    contact = StringField(default='')
    other_sites = StringField(default='')
    education = ListField(ReferenceField(Education))
    experience = ListField(ReferenceField(Experience))
    services = ListField(ReferenceField(Service))
    interests = ListField(ReferenceField("Tag"))
    recommended_books = ListField(ReferenceField("Book"))
    password_reset_token_list = ListField(ReferenceField(PasswordResetToken))
    rating = IntField(default=0)

    def __unicode__(self):
        if self.first_name and self.last_name:
            return u'%s %s' % (self.first_name, self.last_name)
        return u'%s' % self.username

    def get_picture_url(self):
        if self.picture:
            return self.picture.url
        return getattr(settings, 'DEFAULT_PICTURE_URL',
            '/static/images/profile-default.png')

    def check_password(self, raw_password):
        if self.password == md5(raw_password).hexdigest():
            return True
        return False

    def set_password(self, raw_password):
        self.password = md5(raw_password).hexdigest()

    def delete_education(self, education_id):
        try:
            education = Education.objects.get(id=education_id)
            if education in self.education:
                self.update(pull__education=education)
                education.delete()
        except Exception:
            return

    def delete_service(self, service_id):
        try:
            service = Service.objects.get(id=service_id)
            if service in self.services:
                self.update(pull__services=service)
                service.providers_num -= 1
                service.save()
                if not service.providers_num:
                    service.delete()
        except Exception:
            return

    def delete_experience(self, experience_id):
        try:
            experience = Experience.objects.get(id=experience_id)
            if experience in self.experience:
                self.update(pull__experience=experience)
                experience.delete()
        except Exception:
            return

    def delete_recommended_book(self, book_id):
        try:
            recommended_book = Book.objects.get(id=book_id)
            if not recommended_book in self.recommended_books:
                return
            self.update(pull__recommended_books=recommended_book)
            recommended_book.delete()
        except Exception:
            return

    def delete_interest(self, interest_id):
        try:
            interest = Interest.objects.get(id=interest_id)
            if not interest in self.interests:
                return
            self.update(pull__interests=interest)
            interest.delete()
        except Exception:
            return


class Session(Document):
    user = ReferenceField(User)
    session_key = StringField(max_length=64)


class Tag(Document):
    tag = StringField(max_length=64)


class Interest(Tag):
    pass


class Book(Document):
    text = StringField(max_length=1024)
    ref = URLField(required=False)



from datetime import datetime
from md5 import md5
from airy import forms
from airy.core.conf import settings
from os.path import join

from users.auth import authenticate, login
from users.models import User, Session, Service, Experience, Education, Book, Interest


class RegistrationForm(forms.Form):
    username = forms.CharField()
    email = forms.EmailField()
    first_name = forms.CharField()
    last_name = forms.CharField()
    password = forms.CharField(widget=forms.PasswordInput(render_value=True))
    password2 = forms.CharField(widget=forms.PasswordInput(render_value=True), label='Confirm password')

    def clean_username(self):
        if User.objects.filter(username=self.cleaned_data['username']).count() > 0:
            raise forms.ValidationError('A user with such username is already registered')
        return self.cleaned_data['username']

    def clean_email(self):
        if User.objects.filter(email=self.cleaned_data['email']).count() > 0:
            raise forms.ValidationError('A user with such email is already registered')
        return self.cleaned_data['email']

    def clean(self, *args, **kwargs):
        super(RegistrationForm, self).clean(*args, **kwargs)
        if 'password' in self.cleaned_data and 'password2' in self.cleaned_data:
            if self.cleaned_data['password'] != self.cleaned_data['password2']:
                self._errors['password'] = [u'Passwords must match.']
                self._errors['password2'] = [u'Passwords must match']
        return self.cleaned_data

    def save(self, obj):
        user = User(username=self.cleaned_data['username'],
                    first_name=self.cleaned_data['first_name'],
                    last_name=self.cleaned_data['last_name'],
                    email=self.cleaned_data['email'],
                    education=[],
                    experience=[],
                    services=[]
        )
        user.set_password(self.cleaned_data['password'])
        user.save()

        user = authenticate(username=self.cleaned_data['username'], password=self.cleaned_data['password'])

        session_key = md5(self.cleaned_data['username'] + self.cleaned_data['password'])
        obj.set_secure_cookie("session_key", session_key.hexdigest())
        session = Session.objects.get_or_create(user = user)[0]
        session.session_key = session_key.hexdigest()
        session.save()

        return user


class LoginForm(forms.Form):
    username = forms.CharField(label="E-mail")
    password = forms.CharField(widget=forms.PasswordInput(render_value=True))

    def clean(self):
        if 'username' in self.cleaned_data and 'password' in self.cleaned_data:
            username = self.cleaned_data['username']
            if not '@' in username:
                user = authenticate(username=username, password=self.cleaned_data['password'])
            else:
                user = authenticate(email=username, password=self.cleaned_data['password'])
            if not user:
                raise forms.ValidationError('Email or password are incorrect')
        else:
            return None

        self.user = user
        return user

    def save(self, handler):
        login(self.user, self.cleaned_data['password'], handler)
        return self.user


class EducationForm(forms.Form):
    university = forms.CharField(
        widget=forms.TextInput(attrs={'class': 'large input-text'})
    )
    degree = forms.CharField(
        widget=forms.TextInput(attrs={'class': 'large input-text'})
    )
    major = forms.CharField(
        widget=forms.TextInput(attrs={'class': 'large input-text'})
    )
    description = forms.CharField(
        widget=forms.Textarea(attrs={'class': 'post-body'}),
        required=False
    )
    start_date = forms.DateTimeField()
    end_date = forms.DateTimeField(required=False)

    def save(self, handler):
        user = handler.get_current_user()
        education = Education(university=self.cleaned_data['university'],
                              degree=self.cleaned_data['degree'],
                              major=self.cleaned_data['major'],
                              description=self.cleaned_data['description'],
                              start_date=self.cleaned_data['start_date'],
                              end_date=self.cleaned_data['end_date']
        )
        education.save()
        user.education.append(education)
        user.save()
        return education


class ExperienceForm(forms.Form):
    employer = forms.CharField(
        widget=forms.TextInput(attrs={'class': 'large input-text'})
    )
    position = forms.CharField(
        widget=forms.TextInput(attrs={'class': 'large input-text'})
    )
    description = forms.CharField(
        widget=forms.Textarea(attrs={'class': 'post-body'}),
        required=False
    )
    start_date = forms.DateTimeField()
    end_date = forms.DateTimeField(required=False)

    def save(self, handler):
        user = handler.get_current_user()
        experience = Experience(employer=self.cleaned_data['employer'],
                                position=self.cleaned_data['position'],
                                description=self.cleaned_data['description'],
                                start_date=self.cleaned_data['start_date'],
                                end_date=self.cleaned_data['end_date']
        )
        experience.save()
        user.experience.append(experience)
        user.save()
        return experience


class ServicesForm(forms.Form):
    name = forms.CharField(
        widget=forms.TextInput(attrs={'class': 'large input-text'})
    )

    def save(self, handler):
        user = handler.get_current_user()
        service = Service.objects.get_or_create(name=self.cleaned_data['name'])[0]
        service.providers_num += 1
        service.save()
        user.services.append(service)
        user.save()
        return service


class RecommendedBookForm(forms.Form):
    text = forms.CharField(max_length=1024)
    ref = forms.URLField()

    def save(self, handler):
        user = handler.get_current_user()
        book = Book(text=self.cleaned_data['text'], ref=self.cleaned_data['ref'])
        book.save()
        user.recommended_books.append(book)
        user.save()
        return book


class InterestsForm(forms.Form):
    interest = forms.CharField(max_length=64, label='')

    def save(self, handler):
        user = handler.get_current_user()
        interest = Interest(tag=self.cleaned_data['interest'])
        interest.save()
        user.interests.append(interest)
        user.save()
        return interest


class FileUploadForm(forms.Form):
    picture = forms.FileField(label='Choose a file')

    def save(self, user=None, *args, **kwargs):
        file = open(join(settings.static_path, 'media', self.cleaned_data['picture'].name), 'wb')
        file.write(self.cleaned_data['picture'].read())
        file.close()
        if user:
            user.picture_url = '/static/media/%s' % self.cleaned_data['picture'].name
            user.save()
        return '/static/media/%s' % self.cleaned_data['picture'].name


class PasswordRecoveryForm(forms.Form):
    email_or_login = forms.CharField(
        widget=forms.TextInput(attrs={'class': 'large input-text'}),
        label='Please enter your email or password'
    )

    def clean_email_or_login(self):
        data = self.cleaned_data['email_or_login']
        try:
            if '@' in data:
                self.user = User.objects.get(email=data)
            else:
                self.user = User.objects.get(username=data)
        except Exception as e:
            raise forms.ValidationError('A user with such email or username does not exist')
        return self.cleaned_data['email_or_login']

    def save(self):
        return self.user


class NewPasswordForm(forms.Form):
    password = forms.CharField(
        widget=forms.PasswordInput(render_value=True),
        label='Please enter new password'
    )
    password2 = forms.CharField(
        widget=forms.PasswordInput(render_value=True),
        label='Confirm password'
    )

    def clean(self, *args, **kwargs):
        super(NewPasswordForm, self).clean(*args, **kwargs)
        if 'password' in self.cleaned_data and 'password2' in self.cleaned_data:
            if self.cleaned_data['password'] != self.cleaned_data['password2']:
                self._errors['password'] = [u'Passwords must match.']
                self._errors['password2'] = [u'Passwords must match']
        return self.cleaned_data

    def save(self, handler, user):
        user.set_password(self.cleaned_data['password'])
        user.save()
        login(user, self.cleaned_data['password'], handler)
        return user


from handlers import *

urlpatterns = [
    (r"/.*", IndexHandler), # root handler to accept old-style HTTP requests

    (r"/", HomeHandler),
    (r"/accounts/login", AccountsLoginHandler),
    (r"/accounts/logout", AccountsLogoutHandler),
    (r"/accounts/profile", AccountsProfileHandler),
    (r"/accounts/profile/change/", AccountsChangeUserInfoHandler),
    (r"/accounts/profile/delete/", AccountsProfileDeleteHandler),
    (r"/accounts/profile/(?P<object_id>[^\/]+)", AccountsForeignProfileHandler),
    (r"/accounts/profile/picture/(?P<action>\w+)", FileUpload),
    (r"/accounts/recovery/(?P<path>.+)?", AccountsRecoverPasswordHandler),
    (r"/accounts/register", AccountsRegisterHandler),

    # Username handler
    (r"/u/(?P<object_id>\w+)", AccountsForeignProfileHandler),
]





"""
Basic decorators
"""

def bot_friendly(func):

    def wrapped(obj, *args, **kwargs):
        from lxml.html import document_fromstring, tostring, fromstring
        from airy.core.web import site

        def get_current_user():
            return None

        def render(target, template_name, **kwargs):
            document = document_fromstring(obj.HTML)
            fragment = fromstring(handler.render_string(template_name, **kwargs))
            document.get_element_by_id(target[1:]).insert(0, fragment)
            obj.HTML = tostring(document)

        def append(target, data):
            document = document_fromstring(obj.HTML)
            to = document.get_element_by_id(target[1:])
            to.append(fromstring(data))
            obj.HTML = tostring(document)

        def prepend(target, data):
            document = document_fromstring(obj.HTML)
            to = document.get_element_by_id(target[1:])
            to.insert(0, fromstring(data))
            obj.HTML = tostring(document)

        def redirect(url):
            super(obj.__class__, obj).redirect(url)

        def remove(target):
            document = document_fromstring(obj.HTML)
            fragment = document.get_element_by_id(target[1:])
            document.remove(fragment)
            obj.HTML = tostring(document)

        if obj.is_robot():
            obj.HTML = obj.render_string("page.html")
            uri = obj.request.uri
            arguments = obj.request.arguments
            handler, hargs, hkwargs = site.resolve_url(uri, None, arguments)
            handler.get_current_user = get_current_user
            handler.append = append
            handler.redirect = redirect
            handler.remove = remove
            handler.render = render
            handler.prepend = prepend
            handler.get(*hargs, **hkwargs)

            HTML = obj.HTML
            return obj.finish(HTML)
        return func(obj, *args, **kwargs)

    wrapped.__doc__ = func.__doc__
    wrapped.__name__ = func.__name__

    return wrapped



#!/usr/bin/python

from os import path
import sys

PROJECT_ROOT = path.abspath(path.dirname(__file__))

# Airy run engine

def main():
    from airy.core import ve
    ve.check_ve(PROJECT_ROOT, sys.argv)
    from airy.core import manager
    manager.execute(PROJECT_ROOT, sys.argv)

if __name__ == "__main__":
    main()




import smtplib
import thread
import urllib
import urlparse
from datetime import datetime, timedelta
from email.mime.text import MIMEText

from airy.core.web import AiryHandler, AiryRequestHandler
from airy.core.conf import settings

from users.decorators import login_required
from users.forms import *
from users.models import PasswordResetToken


class IndexHandler(AiryRequestHandler):

    def get(self):
        self.render("page.html")


class HomeHandler(AiryHandler):

    def get(self):
        if not self.current_user:
            self.render("#menu", "users/user_out_menu.html")
            self.render("#content", "index.html")
            return
        user = self.get_current_user()
        self.render("#menu", "users/user_in_menu.html")
        self.render("#content", "index.html")

    def post(self):
        if self.current_user:
            self.redirect("/")
            return
        form = RegistrationForm(self.get_flat_arguments())
        if form.is_valid():
            user = form.save(self)
            self.redirect("/")
        else:
            self.render("#menu", "users/user_out_menu.html")
            self.render("#content", "index.html", form=form)


class AccountsLoginHandler(AiryHandler):
    def get(self):
        if self.get_current_user():
            self.redirect("/")
        else:
            form = LoginForm()
            self.render("#content", "users/login.html", form=form)

    def post(self):
        form = LoginForm(self.get_flat_arguments())
        if form.is_valid():
            form.save(self)
            self.redirect("/")
        else:
            self.render("#content", "users/login.html", form=form)


class AccountsLogoutHandler(AiryHandler):
    def get(self):
        self.clear_cookie("session_key")
        self.render("#menu", "users/user_out_menu.html")
        self.redirect("/")


class AccountsRegisterHandler(AiryHandler):
    def get(self):
        form = RegistrationForm()
        self.render("#content", "users/register.html", form=form)

    def post(self):
        form = RegistrationForm(self.get_flat_arguments())
        if form.is_valid():
            user = form.save(self)
            self.redirect("/")
        else:
            self.render("#content", "users/register.html", form=form)


class AccountsProfileHandler(AiryHandler):
    @login_required
    def get(self):
        user = self.get_current_user()
        self.render(
            "#content", "users/profile/main.html",
        )


class AccountsForeignProfileHandler(AiryHandler):
    def get(self, object_id):
        try:
            companies = list()
            try:
                user = User.objects.get(id=object_id)
            except:
                user = User.objects.get(username=object_id)
            self.render(
                "#content", "users/profile/foreign_profile.html",
                user=user, companies=companies, current_user=self.get_current_user()
            )
        except:
            self.render("#content", "http_404.html")


class AccountsChangeUserInfoHandler(AiryHandler):
    @login_required
    def post(self):
        field = self.get_argument('field')
        value = self.get_argument('value')
        user = self.get_current_user()
        setattr(user, field, value)
        user.save()


class AccountsProfileDeleteHandler(AiryHandler):
    @login_required
    def post(self):
        user = self.get_current_user()
        actions = {
            'education': user.delete_education,
            'services': user.delete_service,
            'experience': user.delete_experience,
            'recommended_book': user.delete_recommended_book,
            'interests': user.delete_interest,
        }

        actions[self.get_argument('field')](self.get_argument('object_id'))


class FileUpload(AiryHandler):
    @login_required
    def get(self, action):
        if action == 'upload':
            form = FileUploadForm()
            self.render(
                "#profile-picture",
                "users/profile/picture_upload.html",
                form=form
            )
            self.execute('project.users.picture_init();')
        else:
            self.render("#profile-picture", "users/profile/picture.html")

    @login_required
    def post(self, *args, **kwargs):
        form = FileUploadForm(self.get_flat_arguments(), self.get_files())
        if form.is_valid():
            form.save(self.get_current_user())
            self.render("#profile-picture", "users/profile/picture.html")
            self.render("#menu", "user_in_menu.html")
            return
        self.render(
            "#profile-picture",
            "users/profile/picture_upload.html",
            form=form
        )


class AccountsRecoverPasswordHandler(AiryHandler):
    def get(self, path=None):
        if not path:
            form = PasswordRecoveryForm()
            self.render("#content", "users/password_recovery/recovery.html", form=form)
            return

        args = urlparse.parse_qs(path)
        if not self.token_is_valid(args.get('token', [None])[0]):
            self.redirect("/users/recovery/")

        form = NewPasswordForm()
        self.render("#content", "users/password_recovery/new_password.html", form=form)

    def token_is_valid(self, token):
        try:
            password_reset_token = PasswordResetToken.objects.get(token=token)
            if datetime.now() > password_reset_token.expired:
                return False
            return True
        except Exception:
            return False

    def get_items_from_path(self, path):
        try:
            args = urlparse.parse_qs(path)
            password_reset_token = PasswordResetToken.objects.get(token=args['token'][0])
            user = User.objects.get(password_reset_token_list__contains=password_reset_token)
            return user, password_reset_token
        except Exception:
            return None, None

    def post(self, path=None):
        #Email or username for password recovery, send mail
        if not path:
            form = PasswordRecoveryForm(self.get_flat_arguments())
            if form.is_valid():
                user = form.save()
                thread.start_new_thread(self.send_recovery_mail, (user, self.generate_recovery_link(user)))
                self.render("#content", "users/password_recovery/password_reset_sent.html")
            else:
                self.render("#content", "users/password_recovery/recovery.html", form=form)
            return

        #New password and confirmation
        form = NewPasswordForm(self.get_flat_arguments())

        user, password_reset_token = self.get_items_from_path(path)

        if form.is_valid():
            form.save(self, user)

            #Delete password reset token
            user.update(pull__password_reset_token_list=password_reset_token)
            password_reset_token.delete()

            self.redirect('/')
        else:
            self.render("#content", "users/password_recovery/new_password.html", form=form)


    def generate_recovery_link(self, user):
        expired = datetime.now() + timedelta(days=1)
        token = md5(str(user.id) + expired.strftime("%H:%M:%S %m/%d/%Y")).hexdigest()

        password_reset_token = PasswordResetToken(expired=expired, token=token)
        password_reset_token.save()

        user.password_reset_token_list.append(password_reset_token)
        user.save()

        params = urllib.urlencode(dict(token=token))
        return 'http://%s:%s/users/recovery/%s' % (HOST, PORT, params)

    def send_recovery_mail(self, user, url):
        smtp_server = self.connect_smtp_server(
            settings.email_host,
            settings.email_port,
            settings.email_host_user,
            settings.email_host_password
        )
        smtp_server.sendmail(
            settings.email_host_user,
            user.email,
            self.build_message(user, url)
        )
        smtp_server.quit()

    def build_message(self, user, url):
        msg = MIMEText('Password recovery link: %s' % url)
        msg['Subject'] = 'Password recovery'
        return msg.as_string()

    def connect_smtp_server(self, email_host, email_port, user, password):
        smtp_server = smtplib.SMTP('%s:%s' % (email_host, email_port))
        smtp_server.starttls()
        smtp_server.login(user, password)
        return smtp_server



from airy.core.conf import settings
from airy.core.exceptions import Http404
from airy.core.files.uploadedfile import SimpleUploadedFile
from tornado.web import *
from tornado.escape import *
from tornadio2 import TornadioRouter, SocketConnection, event
from urlparse import urlparse, parse_qs
from urllib2 import unquote
import logging
import base64

class AirySite(object):
    """
    AirySite manages all current Socket.io connections and template loading.
    """
    connections = set()
    application = None
    loader = None

    def resolve_url(self, url, connection, arguments={}):
        handler = None
        args = []
        kwargs = {}

        # unable to check host at the moment, so just loop over all handlers
        for pattern, handlers in self.application.handlers:
            for spec in handlers:
                if issubclass(spec.handler_class, AiryHandler):
                    match = spec.regex.match(url)
                    if match:
                        if spec.regex.groups:
                            # None-safe wrapper around url_unescape to handle
                            # unmatched optional groups correctly
                            def unquote(s):
                                if s is None: return s
                                return escape.url_unescape(s, encoding=None)
                                # Pass matched groups to the handler.  Since
                            # match.groups() includes both named and unnamed groups,
                            # we want to use either groups or groupdict but not both.
                            # Note that args are passed as bytes so the handler can
                            # decide what encoding to use.

                            if spec.regex.groupindex:
                                kwargs = dict(
                                    (k, unquote(v))
                                    for (k, v) in match.groupdict().iteritems())
                            else:
                                args = [unquote(s) for s in match.groups()]
                        handler = spec.handler_class(self, connection, arguments, **spec.kwargs)
                        break

        return handler, args, kwargs

site = AirySite()



class AiryRequestHandler(RequestHandler):
    """
    Old-style handler for plain HTTP requests. Mimics the behaviour
    of Tornado's standard ``RequestHandler`` (from ``tornado.web.RequestHandler``).

    Subclasses are expected to process ordinary HTTP requests.

    An Airy project usually has only one handler which responds to a GET request to any URL
    and is responsible for rendering the base website template, which would establish
    a WebSocket connection.

    All other communication is handled by :py:class:`~airy.core.web.AiryHandler` subclasses.

    For example you may render the default "index" page with an AiryRequestHandler:

    .. code-block:: python

        # snippet from the users app, users/handlers.py:

        class IndexHandler(AiryRequestHandler):

            def get(self):
                self.render("page.html")

    """

    def _flatten_arguments(self, arguments):
        data = {}
        for key, value in arguments.items():
            if len(value) > 1:
                data[key] = value
            else:
                data[key] = value[0]
        return data

    def get_flat_arguments(self):
        """
        Get all arguments as a dictionary where single-value arguments are flat (converted from lists to strings)
        """
        return self._flatten_arguments(self.request.arguments)

    def get_current_user(self):
        """
        Returns a User instance for the current request.

        To fetch current user it loads each backend specified in ``settings.authentication_backends`` and
        calls the ``.get_current_user()`` method of every backend.

        You may specify your own authentication backend in settings and override the default behaviour.

        For more examples refer to the ``users`` app supplied by default with every new project.
        """
        for backend_name in getattr(settings, 'authentication_backends', []):
            backend = __import__(backend_name, fromlist=[backend_name])
            user = backend.get_current_user(self)
            if user:
                return user
        return None

    def get_cookie(self, name, default=None):
        """Gets the value of the cookie with the given ``name``, else ``default``."""
        if self.cookies is not None and name in self.cookies:
            return unquote(self.cookies[name].value)
        return default

    def render_string(self, template_name, **kwargs):
        context_processors = getattr(settings, 'template_context_processors', [])
        template_args = {'reverse_url': self.reverse_url}
        for processor_path in context_processors:
            path, name = processor_path.rsplit('.', 1)
            processor_module = __import__(path, fromlist=path)
            processor = getattr(processor_module, name)
            template_args.update(processor(self, **kwargs))
        template_args.update(kwargs)
        return super(AiryRequestHandler, self).render_string(template_name, **template_args)

    def is_robot(self, *args, **kwargs):
        robots = ['Googlebot', ]
        robots.extend(args)
        for robot in robots:
            if robot in self.request.headers['User-Agent']:
                return True
        return False


class AiryHandler(object):
    """
    Socket.io base handler, responsible for WebSocket communication.

    All (or most of) your handlers should inherit from AiryHandler.

    It emulates the standard HTTP behaviour by providing callbacks for get() and post() requests,
    for example:

    .. code-block:: python

        # snippet from the users app, users/handlers.py:

        class AccountsLoginHandler(AiryHandler):
            def get(self):
                if self.get_current_user():
                    self.redirect("/")
                else:
                    form = LoginForm()
                    self.render("#content", "accounts/login.html", form=form)

            def post(self):
                form = LoginForm(self.get_flat_arguments())
                if form.is_valid():
                    form.save(self)
                    self.redirect("/")
                else:
                    self.render("#content", "accounts/login.html", form=form)

    This is purely for convenience - in reality all data is sent via WebSockets so technically
    a GET request is no different from a POST request. However to simplify form processing and site
    interaction, we send requests to the appropriate method of a handler.

    """

    def __init__(self, site, connection, arguments={}, **kwargs):
        self.site = site
        self.application = self.site.application
        self.connection = connection
        self.arguments = {}
        self.files = {}
        for k, values in arguments.iteritems():
            is_file_content = False
            for v in values:
                if isinstance(v, dict) and 'name' in v and 'content' in v:
                    # this looks like a file
                    # so we process it differently
                    try:
                        mimetype, body = v['content'].split(';', 1)
                        encoding, file_content = body.split(',', 1)
                        if encoding == 'base64':
                            file_content = base64.b64decode(file_content)
                        else:
                            raise RuntimeError('Unknown encoding "%s" specified for field "%s"' % (encoding, k))
                        self.files[k] = SimpleUploadedFile(v['name'], file_content, mimetype)
                        is_file_content = True
                    except Exception, e:
                        logging.error("Failed to decode argument '%s'" % k)
                        continue
            if not is_file_content:
                self.arguments[k] = values
        for k,v in kwargs.iteritems():
            setattr(self, k, v)

    _ARG_DEFAULT = []
    def get_argument(self, name, default=_ARG_DEFAULT, strip=True):
        """Returns the value of the argument with the given name.

        If default is not provided, the argument is considered to be
        required, and we throw an HTTP 400 exception if it is missing.

        If the argument appears in the url more than once, we return the
        last value.

        The returned value is always unicode.
        """
        args = self.get_arguments(name, strip=strip)
        if not args:
            if default is self._ARG_DEFAULT:
                raise HTTPError(400, "Missing argument %s" % name)
            return default
        return args[-1]

    def get_arguments(self, name, strip=True):
        """Returns a list of the arguments with the given name.

        If the argument is not present, returns an empty list.

        The returned values are always unicode.
        """
        values = []
        for v in self.arguments.get(name, []):
            v = self.decode_argument(v, name=name)
            if isinstance(v, unicode):
                # Get rid of any weird control chars (unless decoding gave
                # us bytes, in which case leave it alone)
                v = re.sub(r"[\x00-\x08\x0e-\x1f]", " ", v)
            if strip:
                v = v.strip()
            values.append(v)
        return values

    def _flatten_arguments(self, arguments):
        data = {}
        for key, value in arguments.items():
            if isinstance(value, list):
                if len(value) == 0:
                    data[key] = None
                elif len(value) == 1:
                    data[key] = value[0]
                else:
                    data[key] = value
            else:
                data[key] = value
        return data

    def get_flat_arguments(self):
        arguments = {}
        for key in self.arguments:
            arguments[key] = self.get_arguments(key)
        return self._flatten_arguments(arguments)

    def get_files(self):
        return self.files

    def decode_argument(self, value, name=None):
        """Decodes an argument from the request.

        The argument has been percent-decoded and is now a byte string.
        By default, this method decodes the argument as utf-8 and returns
        a unicode string, but this may be overridden in subclasses.

        This method is used as a filter for both get_argument() and for
        values extracted from the url and passed to get()/post()/etc.

        The name of the argument is provided if known, but may be None
        (e.g. for unnamed groups in the url regex).
        """
        return to_unicode(value)


    @property
    def cookies(self):
        return self.connection.info.cookies

    def get_cookie(self, name, default=None):
        """Gets the value of the cookie with the given name, else default."""
        if self.connection.info.cookies is not None and name in self.connection.info.cookies:
            return unquote(self.connection.info.cookies[name].value)
        return default

    def set_cookie(self, name, value, domain=None, expires=None, path="/",
                   expires_days=None, **kwargs):
        """Sets the given cookie name/value with the given options.

        Additional keyword arguments are set on the Cookie.Morsel
        directly.
        See http://docs.python.org/library/cookie.html#morsel-objects
        for available attributes.
        """
        # The cookie library only accepts type str, in both python 2 and 3
        name = escape.native_str(name)
        value = escape.native_str(value)
        if re.search(r"[\x00-\x20]", name + value):
            # Don't let us accidentally inject bad stuff
            raise ValueError("Invalid cookie %r: %r" % (name, value))
        
        options = {}
        if domain:
            options["domain"] = domain
        if expires_days is not None and not expires:
            expires = datetime.datetime.utcnow() + datetime.timedelta(
                days=expires_days)
        if expires:
            options["expires"] = calendar.timegm(expires.utctimetuple())
        if path:
            options["path"] = path
        for k, v in kwargs.iteritems():
            if k == 'max_age': k = 'max-age'
            options[k] = v

        self.connection.info.cookies[name] = value
        self.execute('airy.set_cookie("%s", "%s", %s);' % (name, value, json_encode(options)))

    def clear_cookie(self, name, path="/", domain=None):
        """Deletes the cookie with the given name."""
        try:
            del self.cookies[name]
        except KeyError:
            pass
        self.execute('airy.set_cookie("%s", null);' % name)

    def clear_all_cookies(self):
        """Deletes all the cookies the user sent with this request."""
        for name in self.connection.info.cookies.iterkeys():
            self.clear_cookie(name)

    def set_secure_cookie(self, name, value, expires_days=30, **kwargs):
        """Signs and timestamps a cookie so it cannot be forged.

        You must specify the ``cookie_secret`` setting in your Application
        to use this method. It should be a long, random sequence of bytes
        to be used as the HMAC secret for the signature.

        To read a cookie set with this method, use `get_secure_cookie()`.

        Note that the ``expires_days`` parameter sets the lifetime of the
        cookie in the browser, but is independent of the ``max_age_days``
        parameter to `get_secure_cookie`.
        """
        self.set_cookie(name, self.create_signed_value(name, value),
                        expires_days=expires_days, **kwargs)

    def create_signed_value(self, name, value):
        """Signs and timestamps a string so it cannot be forged.

        Normally used via set_secure_cookie, but provided as a separate
        method for non-cookie uses.  To decode a value not stored
        as a cookie use the optional value argument to get_secure_cookie.
        """
        self.require_setting("cookie_secret", "secure cookies")
        return create_signed_value(self.application.settings["cookie_secret"],
                                   name, value)

    def get_secure_cookie(self, name, value=None, max_age_days=31):
        """Returns the given signed cookie if it validates, or None."""
        self.require_setting("cookie_secret", "secure cookies")
        if value is None: value = self.get_cookie(name)
        return decode_signed_value(self.application.settings["cookie_secret"],
                                   name, value, max_age_days=max_age_days)

    @property
    def current_user(self):
        """The authenticated user for this request.

        Determined by either get_current_user, which you can override to
        set the user based on, e.g., a cookie. If that method is not
        overridden, this method always returns None.

        We lazy-load the current user the first time this method is called
        and cache the result after that.
        """
        if not hasattr(self, "_current_user"):
            self._current_user = self.get_current_user()
        return self._current_user

    def get_current_user(self):
        for backend_name in getattr(settings, 'authentication_backends', []):
            backend = __import__(backend_name, fromlist=[backend_name])
            user = backend.get_current_user(self)
            if user:
                return user
        return None

    def require_setting(self, name, feature="this feature"):
        """Raises an exception if the given app setting is not defined."""
        if not self.site.application.settings.get(name):
            raise Exception("You must define the '%s' setting in your "
                            "application to use %s" % (name, feature))

    def get_login_url(self):
        """Override to customize the login URL based on the request. By default, we use the 'login_url' application setting."""
        self.require_setting("login_url", "@tornado.web.authenticated")
        return self.site.application.settings["login_url"]

    def reverse_url(self, name, *args):
        """Alias for `Application.reverse_url`."""
        return self.application.reverse_url(name, *args)

    #
    # Airy client interaction
    #
    def execute(self, data):
        """
        Execute JavaScript given in ``data`` on the client side.
        """
        self.connection.emit('execute', data)
        return self

    def redirect(self, url):
        """
        Force client to send a GET request to ``url``.

        This may be used to hand the control over to another handler, for example
         after log in you may send the client to a home page handler:

        .. code-block:: python

            # snippet from users/handlers.py:

            class AccountsRegisterHandler(AiryHandler):

                # ...

                def post(self):
                    form = RegistrationForm(self.get_flat_arguments())
                    if form.is_valid():
                        user = form.save(self)
                        # send the client to the homepage handler:
                        self.redirect("/")


        """
        return self.execute('airy.request("get", "%s");' % url)

    def insert(self, target, data):
        """
        Insert ``data`` into ``target``, where ``target`` is a jQuery selector.

        This is an equivalent to jQuery $.insert() function.
        """
        return self.execute('airy.ui.insert("%s", %s);' % (target, json_encode(data)))

    def append(self, target, data):
        """
        Append ``data`` to ``target``, where ``target`` is a jQuery selector.

        This is an equivalent to jQuery $.append() function.
        """
        return self.execute('airy.ui.append("%s", %s);' % (target, json_encode(data)))

    def prepend(self, target, data):
        """
        Prepend ``data`` to ``target``, where ``target`` is a jQuery selector.

        This is an equivalent to jQuery $.prepend() function.
        """
        return self.execute('airy.ui.prepend("%s", %s);' % (target, json_encode(data)))

    def remove(self, target):
        """
        Remove ``target``, where ``target`` is a jQuery selector.

        This is an equivalent to jQuery $.remove() function.
        """
        return self.execute('airy.ui.remove("%s")' % target)

    def render_string(self, template_name, **kwargs):
        """
        Render the template specified in ``template_name``.

        **Airy** looks up templates in the folder given in ``settings.template_path`` (default: '.templates/')

        We can use render_string() to obtain the output manually before sending it to the client, for example:

        .. code-block:: python

            class MessagesHandler(AiryHandler):

                def get(self):

                    # the client requests a list of all messages

                    # we will iterate over all messages and send each to the client
                    # adding it to the element matching jQuery selection '#content'
                    # (i.e. with id='content')

                    for message in Message.objects.all():

                        output = self.render_string('messages/message.html', message=message)

                        self.prepend(output, '#content')

        Note: if you want to just insert() the output of render_string() into ``target``, just use render() instead.

        """
        context_processors = getattr(settings, 'template_context_processors', [])
        template_args = {'reverse_url': self.reverse_url}
        for processor_path in context_processors:
            path, name = processor_path.rsplit('.', 1)
            processor_module = __import__(path, fromlist=path)
            processor = getattr(processor_module, name)
            template_args.update(processor(self, **kwargs))
        template_args.update(kwargs)
        html = self.site.loader.load(template_name).generate(**template_args)
        return html

    def render(self, target, template_name, **kwargs):
        """
        Render template into the ``target`` element (jQuery selector).

        This is just a handy shortcut, equivalent to:

        .. code-block:: python

            self.insert(
                target,
                self.render_string(template_name, **kwargs)
            )

        You can use ``self.render()`` to send HTML to the client and insert it directly into ``target``.

        For example, you can have an AiryHandler rendering the home page:

        .. code-block:: python

            class HomeHandler(AiryHandler):

                def get(self):
                    self.render("#content", "accounts/index.html")


        """
        html = self.render_string(template_name, **kwargs)
        self.insert(target, html)
        return self


class AiryCoreHandler(SocketConnection):
    "Main Airy handler dealing with WebSocket manipulations"
    state = '/'

    def on_open(self, info):
        self.info = info
        self.site = site
        logging.info('Socket Connected: %s %s' % (self.info.ip, self.info.arguments.get('t', '')))
        site.connections.add(self)

    def set_state(self, state):
        self.state = state

    def parse_url(self, url):
        "Returns a tuple in the form (scheme, host, path, arguments)"
        parsed = urlparse(url)
        arguments = parse_qs(parsed.query)
        return parsed.scheme, parsed.netloc, parsed.path, arguments

    @event('get')
    def get(self, url, *args, **kwargs):
        "Main entry point for WebSocket requests"
        (scheme, host, path, arguments) = self.parse_url(url)
        self.set_state(path)
        handler, hargs, hkwargs = site.resolve_url(path, self, arguments=arguments)
        hargs.extend(args)
        hkwargs.update(kwargs)
        if handler:
            logging.info('GET %s from %s %s => %s' % (path, self.info.ip,
                                                      self.info.arguments.get('t', ''),
                                                      handler.__class__.__name__))
            try:
                handler.get(*hargs, **hkwargs)
            except Http404:
                self.process_404(*hargs, **hkwargs)
        else:
            logging.error('GET %s from %s %s failed: no handler found' % \
                          (path, self.info.ip, self.info.arguments.get('t', '')))
            self.process_404(*hargs, **hkwargs)

    @event('post')
    def post(self, url, *args, **kwargs):
        "Main entry point for WebSocket requests"
        (scheme, host, path, arguments) = self.parse_url(url)
        args = list(args)
        try:
            data = args.pop(0)
            for k, v in dict(data).items():
                if not isinstance(v, list):
                    data[k] = [v]
        except:
            data = {}
        arguments.update(data)
        self.set_state(path)
        handler, hargs, hkwargs = site.resolve_url(path, self, arguments=arguments)
        hargs.extend(args)
        hkwargs.update(kwargs)
        if handler:
            logging.info('POST %s from %s %s => %s' % (path, self.info.ip,
                                                      self.info.arguments.get('t', ''),
                                                      handler.__class__.__name__))
            try:
                handler.post(*hargs, **hkwargs)
            except Http404:
                self.process_404(*hargs, **hkwargs)
        else:
            logging.error('POST %s from %s %s failed: no handler found' % \
                          (path, self.info.ip, self.info.arguments.get('t', '')))
            self.process_404(*hargs, **hkwargs)

    def on_close(self):
        logging.info('Socket Disconnected: %s %s' % (self.info.ip, self.info.arguments.get('t', '')))
        site.connections.remove(self)

    def process_404(self, *args, **kwargs):
        try:
            handler_path = settings.page_not_found_handler
            path, name = handler_path.rsplit('.', 1)
            handler_module = __import__(path, fromlist=path)
            handler_class = getattr(handler_module, name)
            handler_class(site, self).get()
        except AttributeError:
            logging.error('Page with url %s doesn\'t exist' % (kwargs.get('url', ''), ))

    #
    # Airy client interaction
    #
    def execute(self, data):
        "Execute the given data on the client side"
        self.emit('execute', data)
        return self

    def redirect(self, url):
        return self.execute('airy.request("get", "%s");' % url)

    def insert(self, target, data):
        "Insert data into target"
        return self.execute('airy.ui.insert("%s", %s);' % (target, json_encode(data)))

    def append(self, target, data):
        "Append data into target"
        return self.execute('airy.ui.append("%s", %s);' % (target, json_encode(data)))

    def prepend(self, target, data):
        return self.execute('airy.ui.prepend("%s", %s);' % (target, json_encode(data)))

    def render_string(self, template_name, **kwargs):
        "Render the given template"
        context_processors = getattr(settings, 'template_context_processors', [])
        template_args = {'reverse_url': self.reverse_url}
        for processor_path in context_processors:
            path, name = processor_path.rsplit('.', 1)
            processor_module = __import__(path, fromlist=path)
            processor = getattr(processor_module, name)
            template_args.update(processor(self, **kwargs))
        template_args.update(kwargs)
        html = self.site.loader.load(template_name).generate(**template_args)
        return html

    def render(self, target, template_name, **kwargs):
        "Render into the target element (jQuery selector)"
        html = self.render_string(template_name, **kwargs)
        self.insert(target, html)
        return self


core_router = TornadioRouter(AiryCoreHandler)



from airy.core.conf import settings
from mongoengine.queryset import DoesNotExist
from users.auth import get_current_user

def login_required(function):

    def wrapped(handler, *args, **kwargs):
        if get_current_user(handler):
            return function(handler, *args, **kwargs)
        else:
            handler.redirect(getattr(settings, 'login_url', '/accounts/login'))

    wrapped.__doc__ = function.__doc__
    wrapped.__name__ = function.__name__

    return wrapped



"Project settings"

debug = True

database_name = 'airydb' # replace with your Mongo DB name

installed_apps = [
    'users',
]

template_context_processors = [
    'airy.core.context_processors.markup',
    'users.context_processors.settings',
    'users.context_processors.user',
]

login_url = '/accounts/login'

cookie_secret = 'airy_secret' # replace with yours

authentication_backends = [
    'users.auth',
]

datetime_input_formats = (
    '%Y-%m-%d %H:%M:%S',     # '2006-10-25 14:30:59'
    '%Y-%m-%d %H:%M',        # '2006-10-25 14:30'
    '%Y-%m-%d',              # '2006-10-25'
    '%m/%d/%Y %H:%M:%S',     # '10/25/2006 14:30:59'
    '%m/%d/%Y %H:%M',        # '10/25/2006 14:30'
    '%m/%d/%Y',              # '10/25/2006'
    '%m/%d/%y %H:%M:%S',     # '10/25/06 14:30:59'
    '%m/%d/%y %H:%M',        # '10/25/06 14:30'
    '%m/%d/%y',              # '10/25/06'
)

email_host = 'smtp.gmail.com'
email_host_user = ''
email_host_password = ''
email_port = 587



"""
Internationalization support.
"""
import warnings
from os import path

from airy.utils.encoding import force_unicode
from airy.utils.functional import lazy
from airy.utils.importlib import import_module


__all__ = ['gettext', 'gettext_noop', 'gettext_lazy', 'ngettext',
        'ngettext_lazy', 'string_concat', 'activate', 'deactivate',
        'get_language', 'get_language_bidi', 'get_date_formats',
        'get_partial_date_formats', 'check_for_language', 'to_locale',
        'get_language_from_request', 'templatize', 'ugettext', 'ugettext_lazy',
        'ungettext', 'ungettext_lazy', 'pgettext', 'pgettext_lazy',
        'npgettext', 'npgettext_lazy', 'deactivate_all', 'get_language_info']

# Here be dragons, so a short explanation of the logic won't hurt:
# We are trying to solve two problems: (1) access settings, in particular
# settings.USE_I18N, as late as possible, so that modules can be imported
# without having to first configure Django, and (2) if some other code creates
# a reference to one of these functions, don't break that reference when we
# replace the functions with their real counterparts (once we do access the
# settings).

class Trans(object):
    """
    The purpose of this class is to store the actual translation function upon
    receiving the first call to that function. After this is done, changes to
    USE_I18N will have no effect to which function is served upon request. If
    your tests rely on changing USE_I18N, you can delete all the functions
    from _trans.__dict__.

    Note that storing the function with setattr will have a noticeable
    performance effect, as access to the function goes the normal path,
    instead of using __getattr__.
    """

    def __getattr__(self, real_name):
        from airy.core.conf import settings
        if settings.USE_I18N:
            from airy.utils.translation import trans_real as trans
            # Make sure the project's locale dir isn't in LOCALE_PATHS
            if settings.SETTINGS_MODULE is not None:
                parts = settings.SETTINGS_MODULE.split('.')
                project = import_module(parts[0])
                project_locale_path = path.normpath(
                    path.join(path.dirname(project.__file__), 'locale'))
                normalized_locale_paths = [path.normpath(locale_path)
                    for locale_path in settings.LOCALE_PATHS]
                if (path.isdir(project_locale_path) and
                        not project_locale_path in normalized_locale_paths):
                    warnings.warn("Translations in the project directory "
                                  "aren't supported anymore. Use the "
                                  "LOCALE_PATHS setting instead.",
                                  PendingDeprecationWarning)
        else:
            from airy.utils.translation import trans_null as trans
        setattr(self, real_name, getattr(trans, real_name))
        return getattr(trans, real_name)

_trans = Trans()

# The Trans class is no more needed, so remove it from the namespace.
del Trans

def gettext_noop(message):
    return _trans.gettext_noop(message)

ugettext_noop = gettext_noop

def gettext(message):
    return _trans.gettext(message)

def ngettext(singular, plural, number):
    return _trans.ngettext(singular, plural, number)

def ugettext(message):
    return _trans.ugettext(message)

def ungettext(singular, plural, number):
    return _trans.ungettext(singular, plural, number)

def pgettext(context, message):
    return _trans.pgettext(context, message)

def npgettext(context, singular, plural, number):
    return _trans.npgettext(context, singular, plural, number)

ngettext_lazy = lazy(ngettext, str)
gettext_lazy = lazy(gettext, str)
ungettext_lazy = lazy(ungettext, unicode)
ugettext_lazy = lazy(ugettext, unicode)
pgettext_lazy = lazy(pgettext, unicode)
npgettext_lazy = lazy(npgettext, unicode)

def activate(language):
    return _trans.activate(language)

def deactivate():
    return _trans.deactivate()

def get_language():
    return _trans.get_language()

def get_language_bidi():
    return _trans.get_language_bidi()

def get_date_formats():
    return _trans.get_date_formats()

def get_partial_date_formats():
    return _trans.get_partial_date_formats()

def check_for_language(lang_code):
    return _trans.check_for_language(lang_code)

def to_locale(language):
    return _trans.to_locale(language)

def get_language_from_request(request):
    return _trans.get_language_from_request(request)

def templatize(src, origin=None):
    return _trans.templatize(src, origin)

def deactivate_all():
    return _trans.deactivate_all()

def _string_concat(*strings):
    """
    Lazy variant of string concatenation, needed for translations that are
    constructed from multiple parts.
    """
    return u''.join([force_unicode(s) for s in strings])
string_concat = lazy(_string_concat, unicode)

def get_language_info(lang_code):
    from airy.core.conf.locale import LANG_INFO
    try:
        return LANG_INFO[lang_code]
    except KeyError:
        raise KeyError("Unknown language code %r." % lang_code)



import os
from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "airy",
    version = "0.1.1",
    author = "LetoLab Ltd",
    author_email = "team@letolab.com",
    description = ("Web Application Framework"),
    long_description=read('README'),
    license = "BSD",
    keywords = "web development websockets",
    url = "http://airy.letolab.com",
    packages=find_packages(),
    include_package_data=True,
    requires=[
        'simplejson',
        'ipython',
    ],
    scripts=[
        'airy/bin/airy-admin.py',
    ],
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
        "Development Status :: 3 - Alpha",
    ],
)



"""
Configuring Airy projects.

By default Airy parses ``settings.py`` file located in the project's directory.

You may specify parameters either in the file or on command line when launching your
project, for example:

.. code-block:: console

    $ python manage.py runserver --port=80

Most commonly used options:

* port (default=8000)

    Run on the given port

* host (default="127.0.0.1:8000")

    Bind to the given ip/port.

* template_path (default="./templates")

    Path to the directory containing project templates.

* static_path (default="./static")

    Path to your project's static files (Airy will serve it automatically on /static/)

For a full list of options, see ``python manage.py help``

"""
import tornado.options
from tornado.options import define, options
import os

define("port", default=8000, help="Run on the given port", type=int)
define("flash_policy_port", default=8043, help="Port to run Flash policy on (for older browsers)", type=int)
define("flash_policy_file", default="./flashpolicy.xml", help="Flash policy file")
define("secret", default='', help="Application secret")
define("template_path", default="./templates", help="Template directory path")
define("static_path", default="./static", help="Static directory path")
define("app_title", default="Example Web App", help="Default app title")
define("xsrf_cookies", default=True, help="Disable XSRF cookies")
define("installed_apps", default=[], help="List of installed applications", multiple=True)
define("host", default="127.0.0.1:8000")
define("email_host", default="smtp.gmail.com")
define("email_port", default=587)
define("email_host_user", default="")
define("email_host_password", default="")


class Settings(object):
    def __init__(self, **entries):
        self.__dict__.update(entries)

    def __getattribute__(self, name):
        return object.__getattribute__(self, name.lower())
        

settings = Settings()

def _ensure_defaults(dsettings, options, project_root, config_filename):

    # paths    
    if options.template_path == './templates':
        dsettings['template_path'] = os.path.join(project_root, 'templates')
    if options.static_path == './static':
        dsettings['static_path'] = os.path.join(project_root, 'static')
    if options.flash_policy_file == './flashpolicy.xml':
        dsettings['flash_policy_file'] = os.path.join(project_root, 'flashpolicy.xml')
    dsettings['locale_paths'] = ()

    # translation
    dsettings['use_i18n'] = True
    dsettings['use_l10n'] = True
    dsettings['language_code'] = 'en-us'

    # settings
    dsettings['project_root'] = project_root
    dsettings['settings_module'] = config_filename.replace('/', '.')[:-3].strip('.')

    # formats
    dsettings['datetime_format'] = getattr(options, 'datetime_format', 'N j, Y, P')
    dsettings['time_format'] = getattr(options, 'time_format', 'P')

    return dsettings

def _preconfigure(project_root, config_filename='settings.py', argv=[]):
    "Read config file, process command-line args"
    tornado.options.parse_command_line(argv)
    tornado.options.parse_config_file(config_filename)

    dsettings = {}
    for key in options:
        dsettings[key] = getattr(options, key)
    
    _ensure_defaults(dsettings, options, project_root, config_filename)

    config = __import__('settings')
    for name in dir(config):
        if not name.startswith('_'):
            dsettings[name] = getattr(config, name)

    settings.__dict__.update(**dsettings)
    
    settings.socket_io_port = settings.port

    return dsettings





"""
Airy admin script
"""
import sys
import os
import shutil

def execute():

    if len(sys.argv) <= 1:
        print "No command supplied."
        sys.exit(1)

    command = sys.argv[1]
    args = sys.argv[2:]

    if command == 'help':
        help()

    elif command == 'startproject':
        startproject(*args)

    elif command == 'startapp':
        startapp(*args)

    else:
        print "Error: unknown command '%s'" % command

def help():
    print 'Usage: python %s <command>' % sys.argv[0]
    print """
Available commands:

    startproject <project_name>:

        Creates a new project in a folder named <project_name>

    startapp <app_name>:

        Creates a new app in a folder named <app_name>

    help:

        Displays this help

"""

def startproject(project_name):
    admin_path = os.path.abspath(os.path.dirname(__file__))
    skeleton_path = os.path.join(admin_path, '../skeleton/project')
    shutil.copytree(skeleton_path, os.path.join(os.getcwd(), project_name))
    print "Created project '%s'." % project_name

def startapp(app_name):
    admin_path = os.path.abspath(os.path.dirname(__file__))
    skeleton_path = os.path.join(admin_path, '../skeleton/app')
    shutil.copytree(skeleton_path, os.path.join(os.getcwd(), app_name))
    print "Created app '%s'." % app_name



